using System;
using System.Collections;
using System.Diagnostics;
using System.IO;
using EnvDTE;
using VSLangProj;
using System.Runtime.InteropServices;

namespace VSNUnit2k3
{
	/// <summary>
	/// Utility methods for running and debugging projects.
	/// </summary>
	public class ProjectUtils
	{
		/// <summary>
		/// Private constructor for utility class.
		/// </summary>
		private ProjectUtils() {}

		/// <summary>
		/// Debug a program using only the CLR debugger.
		/// </summary>
		/// <param name="project">The project used to debug the program</param>
		/// <param name="program">A path the the program to debug</param>
		/// <param name="workingDirectory">A working directory for the program being debugged</param>
		/// <param name="arguments">Command line arguments of the program being debugged</param>
		public static void DebugProgram(Project project, string program, string workingDirectory, string arguments)
		{
			DebugProgramCommand myDebugProgramCommand = new DebugProgramCommand(project, program, workingDirectory, arguments);
			myDebugProgramCommand.Exec();
		}

		/// <summary>
		/// This class is used to store and restore a projects original debug options.
		/// </summary>
		private class DebugProgramCommand
		{
			private const string START_PROGRAM = "StartProgram";
			private const string START_WORKING_DIRECTORY = "StartWorkingDirectory";
			private const string START_ARGUMENTS = "StartArguments";
			private const string START_ACTION = "StartAction";
			private const string STARTUP_PROJECT = "StartupProject";

			private const string ENABLE_ASP_DEBUGGING = "EnableASPDebugging";
			private const string ENABLE_ASPX_DEBUGGING = "EnableASPXDebugging";
			private const string ENABLE_SQLSERVER_DEBUGGING = "EnableSQLServerDebugging";
			private const string ENABLE_UNMANAGED_DEBUGGING = "EnableUnmanagedDebugging";

			public DebugProgramCommand(Project project, string program, string workingDirectory, string arguments)
			{
				theProject = project;
				theProgram = program;
				theWorkingDirectory = workingDirectory;
				theArguments = arguments;
			}
			private Project theProject;
			private string theProgram;
			private string theWorkingDirectory;
			private string theArguments;

			public void Exec()
			{
				try
				{
					setupConfiguration();
					theDebuggerEvents = theProject.DTE.Events.DebuggerEvents;
					theOnEnterDesignModeEventHandler = new _dispDebuggerEvents_OnEnterDesignModeEventHandler(OnEnterDesignMode);
					theDebuggerEvents.OnEnterDesignMode += theOnEnterDesignModeEventHandler;
					theProject.DTE.Solution.SolutionBuild.Debug();
				}
				catch(Exception)
				{
					restoreConfiguration();
					throw;
				}
			}
			private DebuggerEvents theDebuggerEvents;
			private _dispDebuggerEvents_OnEnterDesignModeEventHandler theOnEnterDesignModeEventHandler;

			public void OnEnterDesignMode(dbgEventReason Reason)
			{
				theDebuggerEvents.OnEnterDesignMode -= theOnEnterDesignModeEventHandler;
				theOnEnterDesignModeEventHandler = null;
				restoreConfiguration();
			}

			private void setupConfiguration()
			{
				Configuration configuration = theProject.ConfigurationManager.ActiveConfiguration;
				theConfigurationProperties = PropertyUtils.SaveProperties(configuration.Properties);
				configuration.Properties.Item(START_PROGRAM).let_Value(theProgram);
				configuration.Properties.Item(START_WORKING_DIRECTORY).let_Value(theWorkingDirectory);
				configuration.Properties.Item(START_ARGUMENTS).let_Value(theArguments);
				configuration.Properties.Item(START_ACTION).let_Value(prjStartAction.prjStartActionProgram);
				configuration.Properties.Item(ENABLE_ASP_DEBUGGING).let_Value("False");
				configuration.Properties.Item(ENABLE_ASPX_DEBUGGING).let_Value("False");
				configuration.Properties.Item(ENABLE_SQLSERVER_DEBUGGING).let_Value("False");
				configuration.Properties.Item(ENABLE_UNMANAGED_DEBUGGING).let_Value("False");
				theStartupProject = (string)theProject.DTE.Solution.Properties.Item(STARTUP_PROJECT).Value;
				theProject.DTE.Solution.Properties.Item(STARTUP_PROJECT).let_Value(theProject.Name);
			}
			private IDictionary theConfigurationProperties;
			private string theStartupProject;

			private void restoreConfiguration()
			{
				Configuration configuration = theProject.ConfigurationManager.ActiveConfiguration;
				if(theConfigurationProperties != null)
				{
					PropertyUtils.LoadProperties(configuration.Properties, theConfigurationProperties);
				}

				if(theStartupProject != null) theProject.DTE.Solution.Properties.Item(STARTUP_PROJECT).let_Value(theStartupProject);
			}
		}

		public static string GetAssemblyPath(Project project)
		{
			try
			{
				string output;
				ConfigurationManager manager = project.ConfigurationManager;
				if(manager == null) return null;

				Configuration configuration = manager.ActiveConfiguration;
				if(configuration == null) return null;

				Properties configurationProperties;
				try	// NOTE: Why does this sometimes throw?
				{
					configurationProperties = configuration.Properties;
				}
				catch(NullReferenceException) { return null; }
				if(configurationProperties == null) return null;

				string myOutputPath = PropertyUtils.GetPropertyString(configurationProperties, "OutputPath");
				if(myOutputPath == null)
				{
					// C++ projects use 'OutputDirectory' rather than 'OutputPath'.
					myOutputPath = PropertyUtils.GetPropertyString(configurationProperties, "OutputDirectory");
					if(myOutputPath == null) return null;
				}

				string myOutputFilename = GetOutputFileName(project);
				if(myOutputFilename == null) return null;

				string myProjectDirectory = GetProjectDirectory(project);

				output = Path.Combine(myProjectDirectory, myOutputPath);
				output = Path.Combine(output, myOutputFilename);
				return output;
			}
			catch(COMException)
			{
				return null;
			}
		}

		/// <summary>
		/// Return the directory the project exists in.
		/// </summary>
		/// <param name="project">A project.</param>
		/// <returns>The directory the project is based in.</returns>
		public static string GetProjectDirectory(Project project)
		{
			return Path.GetDirectoryName(project.FullName);
		}

		/// <summary>
		/// The the output .dll or .exe filename.
		/// </summary>
		/// <param name="aProject">Get the primary output filename for this Project.</param>
		/// <returns>Filename of the primary output .dll or .exe.</returns>
		public static string GetOutputFileName(Project project)
		{
			ConfigurationManager manager = project.ConfigurationManager;
			if(manager == null) return null;
			Configuration configuration = manager.ActiveConfiguration;
			if(configuration == null) return null;

			OutputGroup group = GetOutputGroup(configuration.OutputGroups, "Built");
			if(group != null)
			{
				object[] names = (object[])group.FileNames;
				if(names.Length > 0)
				{
					foreach(string name in names)
					{
						string fileName = Path.GetFileName(name);
						if(fileName.ToLower().EndsWith(".dll") || fileName.ToLower().EndsWith(".exe"))
						{
							return fileName;
						}
					}
				}
			}
			return null;
		}

		public static OutputGroup GetOutputGroup(OutputGroups groups, string canonicalName)
		{
			foreach(OutputGroup group in groups)
			{
				if(group.CanonicalName == canonicalName)
				{
					return group;
				}
			}
			return null;
		}

	}	// End ProjectUtils
}
